#!/usr/bin/python

import tuples
import sys

def main():
    # get strategy from cmd-line parameters
    strategy = sys.argv[1]
    # value to query for
    point_query = int(sys.argv[2])

    # get Blocks (initializes Blocks class)
    blocks = Blocks()

    # execute the strategy requested
    if strategy == "scan":
        result = scan(blocks, point_query)
    elif strategy == "binsearch":
        result = binsearch(blocks, point_query)
    else:
        print "unknown strategy"
        sys.exit(1)

    # output
    print "tuple (%d) found %d times" % (point_query, result)
    print "block accesses: %d" % blocks.get_block_accesses()

class Block:

    #constructor
    def __init__(self, sorted_tuples):
        self.tuples = sorted_tuples

    # get tuple n
    def get_tuple(self, n):
        return self.tuples[n]

    # get number of tuple n
    def get_number_tuples(self):
        return len(self.tuples)

blocksize = 64

class Blocks:
    # constructor
    def __init__(self):
        # get tuples from tuples.py module
        sorted_tuples = tuples.get_tuples()

        # initialize list of blocks
        self.blocks = []

        for i in range(0, len(sorted_tuples), 64):

            # get values for i-th block (we need to be careful because last block is not fully occupied)
            block_tuples = sorted_tuples[i:min(i + blocksize, len(sorted_tuples))]

            # create blocks out of the block_tuples
            block = Block(block_tuples)

            # add block to block list
            self.blocks.append(block)

        # initialize block_accesses counter
        self.block_accesses = 0

    # return block n
    def get_block(self, n):

        # increase block access counter
        self.block_accesses += 1

        # return block n
        return self.blocks[n]

    # return number of blocks
    def get_number_blocks(self):
        return len(self.blocks)

    # return number of block accesses observed
    def get_block_accesses(self):
        return self.block_accesses


# checks block for tuples matching point_query
def check_block(block, point_query):
    count = 0

    # iterate over tuples in block
    for t in range(block.get_number_tuples()):
        
        # increment count if tuple matches query
        if block.get_tuple(t) == point_query:
            count += 1

    return count 


def scan(blocks, point_query):
    count = 0

    # iterate over each block number
    for b in range(blocks.get_number_blocks()):
        
        # get block b
        block = blocks.get_block(b)

        # add number of occurrences of point_query in block to counter
        count += check_block(block, point_query)
        
    return count


def binsearch(blocks, point_query):
    
    ## You should only modify the code in this method
    raise Exception("Not implemented")


# call main
if __name__ == "__main__":
    main()

